package at.kugel.tool.buildtray.config;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.Properties;

/**
 * Config tracks file date and reloads itself.
 *
 * @author <a href="http://www.code-cop.org/">Peter Kofler</a>
 */
public class ReloadingConfig implements Config {

   private final String path;
   private long lastLoaded;
   private Properties props;

   public ReloadingConfig(String configPath) {
      if (configPath == null) {
         throw new IllegalArgumentException("path is null");
      }
      path = configPath;
   }

   @Override
   public String getConfigFileLocation() {
      return path;
   }

   private void checkForUpdate() {
      final long lastModified = configLastModified();
      if (lastLoaded < lastModified) {
         reloadPropertiesHandleErrors();
         lastLoaded = lastModified;
      }
   }

   private long configLastModified() {
      return new File(path).lastModified();
   }

   private void reloadPropertiesHandleErrors() {
      try {
         reloadProperties();
      } catch (IOException e) {
         throw new IllegalStateException(e);
      }
   }

   private void reloadProperties() throws IOException {
      props = new Properties();
      final Reader in = new FileReader(path);
      try {
         props.load(in);
      } finally {
         in.close();
      }
   }

   @Override
   public long getPollingMs() {
      return getInteger("polling.ms");
   }

   private int getInteger(String key) {
      return Integer.parseInt(getValue(key));
   }

   private String getValue(String key) {
      checkForUpdate();
      return props.getProperty(key);
   }

   @Override
   public String getServerlUrl() {
      return getValue("server.url");
   }

   @Override
   public String getServerDisplayUrl() {
      return getValue("server.display");
   }

   @Override
   public boolean isCheckBusy() {
      return getBoolean("busy.check");
   }

   private boolean getBoolean(String key) {
      return Boolean.valueOf(getValue(key));
   }

   @Override
   public String getDatePattern() {
      return getValue("status.date.format");
   }

   @Override
   public long getSplashWaitMs() {
      return getInteger("splash.wait.ms");
   }

   @Override
   public String getBusyPattern() {
      return getValue("busy.pattern");
   }

   @Override
   public String getSuccessValue() {
      return getValue("value.success");
   }

   @Override
   public String getFailedValue() {
      return getValue("value.failed");
   }

   @Override
   public String getStatusPattern() {
      return getValue("status.pattern");
   }

   @Override
   public int getStatusPatternNameGroup() {
      return getInteger("status.name.group");
   }

   @Override
   public int getStatusPatternStatusGroup() {
      return getInteger("status.value.group");
   }

   @Override
   public int getStatusPatternDateGroup() {
      return getInteger("status.date.group");
   }

   @Override
   public String getProxyHost() {
      return getValue("proxy.host");
   }

   @Override
   public int getProxyPort() {
      return getInteger("proxy.port");
   }

   @Override
   public boolean useProxy() {
      return getBoolean("proxy.set");
   }

   @Override
   public boolean useAuth() {
      return getBoolean("server.basic.auth");
   }

   @Override
   public String getServerUsername() {
      return getValue("server.username");
   }

   @Override
   public String getServerPassword() {
      return getValue("server.password");
   }

   @Override
   public String getTrustStore() {
      return getValue("server.trust.store.location");
   }

   @Override
   public String getTrustStorePassword() {
      return getValue("server.trust.store.password");
   }

   @Override
   public boolean useTrustStore() {
      return getBoolean("server.trust.store");
   }

}
